#
# $Id: converter.rb 28 2005-01-13 00:51:17Z thomas $
#
#--
#
# xmlresume2x: converts an xml resume to various output formats
# Copyright (C) 2004 Thomas Leitner
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU
# General Public License as published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
# even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program; if not,
# write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#++
#

require 'rbconfig'
require 'erb'
require 'rexml/document'
require 'xmlresume2x/processor'

module XMLResume2x

  VERSION = [0, 2, 1]

  class Converter

    CONFIG_EXT = '.cfg'

    attr_accessor :data_dir

    def initialize
      @data_dir = File.join( Config::CONFIG["datadir"], "xmlresume2x" )
      if defined? Gem::Cache
        gem = Gem::Cache.from_installed_gems.search( "xmlresume2x", "=#{VERSION.join('.')}" ).last
        @data_dir = File.join( gem.full_gem_path, "data", "xmlresume2x" ) if gem
      end
    end

    def available_formats
      dir_list( format_dir )
    end

    def available_langs
      dir_list( lang_dir )
    end

    def load_config( format, lang, userconfig = nil )
      @processor = ResumeProcessor.new( lang )

      filename = File.join( format_dir, format + CONFIG_EXT )
      @processor.load_config( File.read( filename ), filename)

      filename = File.join( lang_dir, lang + CONFIG_EXT )
      @processor.load_config( File.read( filename ), filename )

      @processor.load_config( File.read( userconfig ), userconfig ) if userconfig
    end

    def convert( resume )
      ElementWrapper.new( REXML::Document.new( File.new( resume ) ).root, @processor, {} ).process!
    end

    private

    def format_dir
      File.join( @data_dir, 'format' )
    end

    def lang_dir
      File.join( @data_dir, 'lang' )
    end

    def dir_list( dir )
      Dir[File.join( dir, '*' + CONFIG_EXT )].collect {|name| File.basename( name, CONFIG_EXT ) }
    end

  end

end
