#
# $Id: wrapper.rb 27 2005-01-12 20:17:13Z thomas $
#
#--
#
# xmlresume2x: converts an xml resume to various output formats
# Copyright (C) 2004 Thomas Leitner
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU
# General Public License as published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
# even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program; if not,
# write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#++
#

class Array

  alias_method :old_join, :join
  def join( sep = $, )
    if sep.instance_of?( String ) || sep.instance_of?( NilClass )
      self.collect {|i| i.to_s}.old_join( sep )
    else
      arr = []
      self.each_with_index {|e, i| arr << sep.clone if i > 0; arr << e}
      arr
    end
  end

  def separate_by( sep, sepLast )
    result = self[0..-2].join( sep )
    case self.length
    when 0 then result
    when 1 then self.last
    when 2
      if result.kind_of? String
        result << sepLast.to_s << self.last.to_s
      else
        result << sepLast << self.last
      end
    end
  end

end

module XMLResume2x

  class ElementWrapperList < Array

    def method_missing( name, *args )
      return nil if self.length == 0
      self[0].send( name.to_s, *args )
    end

    def process_all!( options = {} )
      self.collect {|e| e.process!( options ) }
    end

  end

  class ElementWrapper

    attr_reader :options

    def initialize( element, processor, options )
      @element = element
      @processor = processor
      @options = options
    end

    def method_missing( name, *args )
      name = name.to_s
      if name =~ /^_/
        name.sub!( /^_/, '' )
        @processor.get_value( @element.attributes[name], @options )
      elsif @element.elements[name]
        list = ElementWrapperList.new
        @element.each_element( name ) {|e| list << ElementWrapper.new( e, @processor, @options ) }
        list
      else
        nil
      end
    end

    def __element
      @element
    end

    def process!( options = {} )
      @processor.get_value( self, options )
    end

    def process_children!( options = {} )
      @processor.get_children_value( self, options )
    end

    def text!
      @processor.get_value( @element.text, @options )
    end

    def to_s
      process!.to_s
    end

  end

end
