#
# $Id: xmlmarkup.rb 22 2004-12-10 18:21:39Z thomas $
#
#--
#
# xmlresume2x: converts an xml resume to various output formats
# Copyright (C) 2004 Thomas Leitner
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU
# General Public License as published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
# even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program; if not,
# write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#++
#


class XMLResume2x::ResumeProcessor

  def tag( name, attr = nil, context = nil )
    e = REXML::Element.new( name, context )
    attr.each {|key, value| e.add_attribute( key, value )} if attr
    e
  end

end

class REXML::Parent

  alias_method :old_add, :add

  def add( object, *others )
    if object.instance_of?( XMLResume2x::ElementWrapper ) || object.instance_of?( XMLResume2x::ElementWrapperList )
      add( object.process! )
    elsif object.instance_of? XMLResume2x::REXMLBuilder
      add( object.element! )
    elsif object.instance_of? Array
      object.each {|o| add( o )}
    elsif object.instance_of? String
      old_add( REXML::Text.new( object ) )
    else
      old_add( object )
    end
    others.each {|o| add( o )}
    self
  end

end

class REXML::Element

  def builder
    XMLResume2x::REXMLBuilder.new( self )
  end

end


class XMLResume2x::REXMLBuilder

  instance_methods.each {|m| undef_method( m ) unless m =~ /^(__|instance_eval)|\?$/}

  def initialize( element )
    @element = element
  end

  def method_missing( name, *args, &block )
    attrs = {}
    content = nil
    args.each do |a|
      if a.instance_of? Hash
        attrs.merge!( a )
      else
        content ||= []
        content << a
      end
    end
    new_elem = @element.add_element( name.to_s, attrs )
    new_elem.add( content ) unless content.nil?
    yield XMLResume2x::REXMLBuilder.new( new_elem ) if block_given?
    self
  end

  def text!( str )
    @element.add_text( str )
    self
  end

  def add!( object, *others )
    @element.add( object, *others )
    self
  end

  def element!
    @element
  end

end
